//______________________________________________________________________________
// SliceSymmetrizePlugin
// 10/2010 by Eugen Sares
// last revision: 26.10.2010
//
// This Command mimics the 3ds max Symmetry Modifier.
// First it creates a Curve Rectangle of the size of the selection BBox,
// then applies a Slice and a Symmetrize Op to the selected polymeshes
// and links their Custom Planes to the Rectangle.
//
// The Slice Operator does have a convenient option to link it's Custom Plane to any object,
// but the Symmetrize Op does not.
// Workaround: SCOPs are put on the Symm Op's Custom Plane's posx, posy, posz, rotx, roty, rotz
// parameters that calculate the local space of the Plane.
//______________________________________________________________________________

function XSILoadPlugin( in_reg )
{
	in_reg.Author = "Eugen";
	in_reg.Name = "SliceSymmetrizePlugin";
	in_reg.Major = 1;
	in_reg.Minor = 0;

	in_reg.RegisterCommand("SliceSymmetrize","SliceSymmetrize");
	in_reg.RegisterMenu(siMenuTbModelModifyPolygonMeshID,"SliceSymmetrize_Menu",false,false);
	//RegistrationInsertionPoint - do not remove this line

	return true;
}

//______________________________________________________________________________

function XSIUnloadPlugin( in_reg )
{
	var strPluginName;
	strPluginName = in_reg.Name;
	Application.LogMessage(strPluginName + " has been unloaded.",siVerbose);
	return true;
}

//______________________________________________________________________________

function SliceSymmetrize_Init( in_ctxt )
{
	var oCmd;
	oCmd = in_ctxt.Source;
	oCmd.Description = "";
	oCmd.ReturnValue = true;

	return true;
}

//______________________________________________________________________________

function SliceSymmetrize_Execute(  )
{

	Application.LogMessage("SliceSymmetrize_Execute called",siVerbose);
	
	// get Bounding Box of all selected Objects
	var cSel = Application.Selection;
	
	try {
		var cPolyMeshes = SIFilter(cSel, "polygon_mesh", true);
		// Tip: SIFilter does not return a valid empty collection.
		// If the collection is empty, an error will be thrown when accessing the Count property.

		if(cPolyMeshes.Count > 0)
		{	
			var aRtn = GetBBox(cPolyMeshes);
			var xmin = aRtn.value( "LowerBoundX" );
			var ymin = aRtn.value( "LowerBoundY" );
			var zmin = aRtn.value( "LowerBoundZ" );
			var xmax = aRtn.value( "UpperBoundX" );
			var ymax = aRtn.value( "UpperBoundY" );
			var zmax = aRtn.value( "UpperBoundZ" );

			var planeX = (xmax + xmin) / 2;
			var planeY = (ymax + ymin) / 2;
			var planeZ = (zmax + zmin) / 2;

			// create a quare curve helper object
			var oPlane = SICreateCurve("plane", 1, 1);
			SIAddPointOnCurveAtEnd(oPlane, xmax - planeX, 0, zmax - planeZ, false, 0, null);
			SIAddPointOnCurveAtEnd(oPlane, xmin - planeX, 0, zmax - planeZ, false, 0, null);
			SIAddPointOnCurveAtEnd(oPlane, xmin - planeX, 0, zmin - planeZ, false, 0, null);
			SIAddPointOnCurveAtEnd(oPlane, xmax - planeX, 0, zmin - planeZ, false, 0, null);
			ApplyTopoOp("CrvOpenClose", oPlane, 3, siPersistentOperation, null);

			// set the Plane to the COG of the selected object's bounding box
			oPlane.PosX = planeX;
			oPlane.PosY = planeY;
			oPlane.PosZ = planeZ;

			//oPlane.SclX = xmax - xmin;
			//oPlane.SclY = ymax - ymin;
			//oPlane.SclZ = zmax - zmin;

			// set wire color of Plane to yellow
			MakeLocal(oPlane + ".display", siDefaultPropagation);
			oDisplay = oPlane.Properties("display");
			oDisplay.Parameters("wirecolorr").Value = 0.878;
			oDisplay.Parameters("wirecolorg").Value = 0.878;
			oDisplay.Parameters("wirecolorb").Value = 0;


			//logMatrix4(PLANE);

			// loop n through all selected Objects
			for(var n = 0; n < cPolyMeshes.Count; n++)
			{
				var oObject = cPolyMeshes(n);

				// Slice Operator
				var oSliceOp = ApplyTopoOp("SlicePolygons", oObject, siUnspecified, siPersistentOperation, null);
				SetValue(oSliceOp + ".planeref", 1, null);
				SetValue(oSliceOp + ".sliceplane", 3, null);
				SetValue(oSliceOp + ".sliceaction", 2, null);
				SetValue(oSliceOp + ".syncwithtransfo", true, null);
				ConnectPlaneReferenceToSlicePolygonsOp(oSliceOp + ".sliceplane", oPlane);

				// Symmetrize Operator
				var oSymmOp = ApplyTopoOp("SymmetrizePolygon", oObject, siUnspecified, siPersistentOperation, null);
				SetValue(oSymmOp + ".symmetryreference", 1, null);
				SetValue(oSymmOp + ".symmetryplane", 3, null);
				SetValue(oSymmOp + ".planenormal", 1, null);
				SetValue(oSymmOp + ".weld", true, null);
				SetValue(oSymmOp + ".weldonlysym", true, null);

				// create Collection of inputs to SCOP (Object's and Plane's kine.global)
				var col = XSIFactory.CreateActiveXObject( "XSI.Collection" );
				col.Add(oObject.Kinematics.Global);
				col.Add(oPlane.Kinematics.Global);

				// add the SCOP to all 6 params of Symmetrize Operator's Custom Plane
				AddScriptedOp(oSymmOp + ".planeposx", symmPlane_Update.toString(), col, "symmPlane", "JScript", 0);
				AddScriptedOp(oSymmOp + ".planeposy", symmPlane_Update.toString(), col, "symmPlane", "JScript", 0);
				AddScriptedOp(oSymmOp + ".planeposz", symmPlane_Update.toString(), col, "symmPlane", "JScript", 0);
				AddScriptedOp(oSymmOp + ".planerotx", symmPlane_Update.toString(), col, "symmPlane", "JScript", 0);
				AddScriptedOp(oSymmOp + ".planeroty", symmPlane_Update.toString(), col, "symmPlane", "JScript", 0);
				AddScriptedOp(oSymmOp + ".planerotz", symmPlane_Update.toString(), col, "symmPlane", "JScript", 0);

			} // end for
			
		} // end if
		
		return true;
	
	}
	catch(e)
	{
		Application.LogMessage("Please select some polymesh objects first.", siWarning);
	}
	
	return false;
}

//______________________________________________________________________________

// SCOP function
function symmPlane_Update(ctx, Out, inglobal1, inglobal2)
{
	var transfo1 = inglobal1.Value.Transform;
	var OBJECT = transfo1.Matrix4;	// get Matrix4 of this object

	var transfo2 = inglobal2.Value.Transform;
	var PLANE = transfo2.Matrix4;	// get Matrix4 of symmetry plane

	// The SymmOp's Custom Plane has to be in it's Object's space,
	// so we need to multiply the Plane matrix with the inverse Object matrix.
	OBJECT.Invert(OBJECT);
	PLANE.Mul(PLANE, OBJECT);
	var symmTransfo = XSIMath.CreateTransform();
	symmTransfo.SetMatrix4(PLANE);

	if(Out.Name == "Outplaneposx") Out.Value = symmTransfo.PosX;
	else if (Out.Name == "Outplaneposy") Out.Value = symmTransfo.PosY;
	else if (Out.Name == "Outplaneposz") Out.Value = symmTransfo.PosZ;
	else if (Out.Name == "Outplanerotx") Out.Value = symmTransfo.RotX;
	else if (Out.Name == "Outplaneroty") Out.Value = symmTransfo.RotY;
	else if (Out.Name == "Outplanerotz") Out.Value = symmTransfo.RotZ;
}

//______________________________________________________________________________

function SliceSymmetrize_Menu_Init( in_ctxt )
{
	var oMenu;
	oMenu = in_ctxt.Source;
	oMenu.AddCommandItem("SliceSymmetrize","SliceSymmetrize");
	return true;
}

//______________________________________________________________________________
